@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2 of the License, or (at your
+ * Free Software Foundation; either version 3 of the License, or (at your
  * option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
@@ -21,10 +21,16 @@
 
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
 #include <string.h>
 
+#if GTK_CHECK_VERSION (2, 90, 7)
+#define GDK_KEY(symbol) GDK_KEY_##symbol
+#else
+#include <gdk/gdkkeysyms.h>
+#define GDK_KEY(symbol) GDK_##symbol
+#endif
+
 #include "gucharmap-charmap.h"
 #include "gucharmap-unicode-info.h"
 #include "gucharmap-marshal.h"
@@ -72,7 +78,8 @@ enum {
   PROP_ACTIVE_CODEPOINT_LIST,
   PROP_ACTIVE_PAGE,
   PROP_SNAP_POW2,
-  PROP_FONT_DESC
+  PROP_FONT_DESC,
+  PROP_FONT_FALLBACK
 };
 
 static guint gucharmap_charmap_signals[NUM_SIGNALS];
@@ -134,6 +141,9 @@ gucharmap_charmap_get_property (GObject *object,
     case PROP_FONT_DESC:
       g_value_set_boxed (value, gucharmap_charmap_get_font_desc (charmap));
       break;
+    case PROP_FONT_FALLBACK:
+      g_value_set_boolean (value, gucharmap_charmap_get_font_fallback (charmap));
+      break;
     case PROP_SNAP_POW2:
       g_value_set_boolean (value, gucharmap_charmap_get_snap_pow2 (charmap));
       break;
@@ -174,6 +184,9 @@ gucharmap_charmap_set_property (GObject *object,
     case PROP_FONT_DESC:
       gucharmap_charmap_set_font_desc (charmap, g_value_get_boxed (value));
       break;
+    case PROP_FONT_FALLBACK:
+      gucharmap_charmap_set_font_fallback (charmap, g_value_get_boolean (value));
+      break;
     case PROP_SNAP_POW2:
       gucharmap_charmap_set_snap_pow2 (charmap, g_value_get_boolean (value));
       break;
@@ -320,6 +333,20 @@ gucharmap_charmap_class_init (GucharmapCharmapClass *klass)
                          G_PARAM_STATIC_NICK |
                          G_PARAM_STATIC_BLURB));
 
+  /**
+   * GucharmapCharmap:font-fallback:
+   *
+   * Whether font fallback is enabled.
+   *
+   * Since: 2.34
+   */
+  g_object_class_install_property
+    (object_class,
+     PROP_FONT_FALLBACK,
+     g_param_spec_boolean ("font-fallback", NULL, NULL,
+                           TRUE,
+                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   g_object_class_install_property
     (object_class,
      PROP_SNAP_POW2,
@@ -937,9 +964,9 @@ details_key_press_event (GtkWidget *text_view,
 
   switch (event->keyval)
     {
-      case GDK_Return:
-      case GDK_ISO_Enter:
-      case GDK_KP_Enter:
+      case GDK_KEY (Return):
+      case GDK_KEY (ISO_Enter):
+      case GDK_KEY (KP_Enter):
         buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
         gtk_text_buffer_get_iter_at_mark (buffer, &iter, 
                                           gtk_text_buffer_get_insert (buffer));
@@ -1049,7 +1076,7 @@ details_motion_notify_event (GtkWidget *text_view,
 
   set_cursor_if_appropriate (charmap, x, y);
 
-  gdk_window_get_pointer (text_view->window, NULL, NULL, NULL);
+  gdk_window_get_pointer (gtk_widget_get_window (text_view), NULL, NULL, NULL);
   return FALSE;
 }
 
@@ -1060,7 +1087,7 @@ details_visibility_notify_event (GtkWidget *text_view,
 {
   gint wx, wy, bx, by;
 
-  gdk_window_get_pointer (text_view->window, &wx, &wy, NULL);
+  gdk_window_get_pointer (gtk_widget_get_window (text_view), &wx, &wy, NULL);
 
   gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW (text_view), 
                                          GTK_TEXT_WINDOW_WIDGET,
@@ -1073,7 +1100,11 @@ details_visibility_notify_event (GtkWidget *text_view,
 
 static void
 notebook_switch_page (GtkNotebook *notebook,
-                      GtkNotebookPage *page /* useless */,
+#if GTK_CHECK_VERSION (2, 90, 6)
+                      GtkWidget *page,
+#else
+                      gpointer page /* useless */,
+#endif
                       guint page_num,
                       GucharmapCharmap *charmap)
 {
@@ -1349,6 +1380,41 @@ gucharmap_charmap_get_font_desc (GucharmapCharmap *charmap)
   return charmap->priv->font_desc;
 }
 
+/**
+ * gucharmap_charmap_set_font_fallback:
+ * @charmap: a #GucharmapCharmap
+ * @enable_font_fallback: whether to enable font fallback
+ *
+ * Since: 2.34
+ */
+void 
+gucharmap_charmap_set_font_fallback (GucharmapCharmap *charmap,
+                                     gboolean enable_font_fallback)
+{
+  g_return_if_fail (GUCHARMAP_IS_CHARMAP (charmap));
+
+  gucharmap_chartable_set_font_fallback (charmap->priv->chartable, 
+                                         enable_font_fallback);
+
+  g_object_notify (G_OBJECT (charmap), "font-fallback");
+}
+
+/**
+ * gucharmap_charmap_get_font_fallback:
+ * @charmap: a #GucharmapCharmap
+ *
+ * Returns: whether font fallback is enabled
+ *
+ * Since: 2.34
+ */
+gboolean
+gucharmap_charmap_get_font_fallback (GucharmapCharmap *charmap)
+{
+  g_return_val_if_fail (GUCHARMAP_IS_CHARMAP (charmap), FALSE);
+
+  return gucharmap_chartable_get_font_fallback (charmap->priv->chartable);
+}
+
 void
 gucharmap_charmap_set_active_character (GucharmapCharmap *charmap,
                                         gunichar          wc)
@@ -1482,7 +1548,11 @@ gucharmap_charmap_get_chapters_visible (GucharmapCharmap *charmap)
 {
   GucharmapCharmapPrivate *priv = charmap->priv;
 
+#if GTK_CHECK_VERSION (2,18,0)
+  return gtk_widget_get_visible (GTK_WIDGET (priv->chapters_view));
+#else
   return GTK_WIDGET_VISIBLE (priv->chapters_view);
+#endif
 }
 
 void
@@ -1511,7 +1581,11 @@ gucharmap_charmap_get_page_visible (GucharmapCharmap *charmap,
   if (!page_widget)
     return FALSE;
 
+#if GTK_CHECK_VERSION (2,18,0)
+  return gtk_widget_get_visible (page_widget);
+#else
   return GTK_WIDGET_VISIBLE (page_widget);
+#endif
 }
 
 void
